#!/bin/bash

me=$(readlink -f $0)
BASE=$(dirname ${me})

OS=$(if [ -e /etc/redhat-release ]; then cat /etc/redhat-release | head -n 1 | cut -d " " -f 1; else if [ -r /etc/issue ]; then cat /etc/issue | head -n 1 | cut -d " " -f 1; else echo unknown; fi; fi)

if [ "${OS}" != "CentOS" ]; then
    read -p "Wrong OS! [Abort]" answer
    exit 2
fi

OSVersion=$(if [ -e /etc/redhat-release ]; then cat /etc/redhat-release | head -n 1 | cut -d " " -f 4 | cut -d "." -f 1,2; else if [ -r /etc/issue ]; then cat /etc/issue | head -n 1 | cut -d " " -f 3; else echo unknown; fi; fi)

if ! ls ${BASE}/*.rpm | grep -q $(uname -p); then
    read -p "GOS Version \"${OS} ${OSVersion}\" has wrong architecure. Please update system via USB stick or call the Ferrari electronic hotline. [Abort]" answer
    exit 3
fi

architecture=$(uname -p)
if [ "${architecture}" == "x86_64" ]; then
    if [[ ${OSVersion:0:1} != "8" ]]; then
        read -p "OS Version \"${OS} ${OSVersion}\" is not supported. Please update system via USB stick or call the Ferrari electronic hotline. [Abort]" answer
        exit 4
    fi
fi

featureversion=$(featureversion)

if [ ! -z "${featureversion}" ]; then
    if [ -e /tmp/gateconfig.featureversion ]; then
        gateconfigversion=$(cat /tmp/gateconfig.featureversion)
        if [ -z "${gateconfigversion}" -o ${gateconfigversion} -lt ${featureversion} ]; then
            echo "***********************************************************************************"
            echo "* OfficeMaster GateConfiguration too old. Please download new version"
            echo "* http://www.ferrari-electronic.de/fileadmin/download_center/OfficeMasterHardware/Setup-OfficeMaster-Gate-Config.exe"
            echo "***********************************************************************************"
            read -p "OfficeMaster GateConfiguration too old. Please download new version. [Abort/Default=Abort]!" answer
            sleep 5
            echo "## Installation failed ## ..."
            echo "FLASH_FAILED" # Old gateConfig
            touch /tmp/flashfailed
            exit 1
        fi
    else
        if [ -e /var/config.active -o "${HISTFILE}" != "" ]; then
            echo "***********************************************************************************"
            echo "* OfficeMaster GateConfiguration too old. Please download new version"
            echo "* http://www.ferrari-electronic.de/fileadmin/download_center/OfficeMasterHardware/Setup-OfficeMaster-Gate-Config.exe"
            echo "***********************************************************************************"
            read -p "OfficeMaster GateConfiguration too old. Please download new version. [Abort/Default=Abort]!" answer
            sleep 5
            echo "## Installation failed ## ..."
            echo "FLASH_FAILED" # Old gateConfig
            touch /tmp/flashfailed
            exit 2
        fi
    fi
fi

function checkflashit() {
    OSType=$(bash -c "if [ -r /etc/redhat-release ] ; then cat /etc/redhat-release | head -n 1 | cut -d ' ' -f 1 | sed 's/.*/\L&/' ; elif [ -r /etc/issue ] ; then cat /etc/issue | head -n 1 | cut -d ' ' -f 1 | sed 's/.*/\L&/'; else echo unknown ; fi")

    if [ $OSType = "centos" ]; then
        echo "CentOS detected"
    else
        echo "unknown OS"
        return 3
    fi

    /usr/bin/omgstartilid

    for i in $(getchannels -a 2>&1 | grep '^0\.[a-z]/X/CFG'); do
        hw=$(echo $i | cut -d '.' -f 2 | cut -d '/' -f 1)
        case "${hw}" in
        "a")
            device="/dev/tdmSpi8"
            ;;
        "b")
            device="/dev/tdmSpi108"
            ;;
        "c")
            device="/dev/tdmSpi208"
            ;;
        "d")
            device="/dev/tdmSpi308"
            ;;
        "e")
            device="/dev/tdmSpi408"
            ;;
        "f")
            device="/dev/tdmSpi508"
            ;;
        "g")
            device="/dev/tdmSpi608"
            ;;
        "h")
            device="/dev/tdmSpi708"
            ;;
        *)
            echo "cant calculate apropiate device!"
            return 3
            ;;
        esac

        let version=0x$(echo fpga_version | confTool /dev/ilid ${i} | head -1 | cut -f 3 -d ' ')
        if [ -z "${version}" ]; then
            echo "Error: Cannot detect fpga version."
            return 1
        fi

        if [ "${isS2Me}" == "1" ]; then
            echo "it's an S2Me"
            let versiontoflash=0x0109
            let minversiontoflash=0x0106
        elif [ "${isQuadE1}" = "1" ]; then
            echo "it's an 4S2M"
            let versiontoflash=0x010a
            let minversiontoflash=0x010a
        elif [ "${isAtomBox}" == "1" ]; then
            echo "it's an OMGA"
            let versiontoflash=0x010c
            let minversiontoflash=0x010c
        else
            echo "nothing to flash"
            return 0
        fi

        #printf "            version is: %04X\n" ${version}
        #printf "   version to flash is: %04X\n" ${versiontoflash}
        #printf "good enough version is: %04X\n" ${minversiontoflash}

        if [ -z "${version}" -o ${version} -lt ${minversiontoflash} ]; then
            flashit=1
        fi
    done

    if [ "${flashit}" == "1" ]; then
        read -p "<notify>You need an FPGA update. After update it is absolutely necessary to powercycle your box. [Continue/Abort/Default=Abort]?" answer
        if [ "${answer}" != "Continue" -a "${answer}" != "C" -a "${answer}" != "c" ]; then
            echo "Aborting"
            return 1
        fi
    fi

    return 0
}

echo "Check hardware for FPGA update ..."

isS2Me=$(if lspci -n | grep -q "1011:f001"; then echo 1; else echo 0; fi)
isQuadE1=$(if lspci -n | grep -q "1011:f004"; then echo 1; else echo 0; fi)
isAtomBox=$(if lspci -n | grep -q "1011:f005"; then echo 1; else echo 0; fi)

if [ "${isS2Me}" != "1" -a "${isQuadE1}" != "1" -a "${isAtomBox=}" != "1" ]; then
    echo "Hardware without FPGA."
else
    checkflashit
    sleep 10
fi

rpmlist=$(find ${BASE} -name 'omg-watchdog*.rpm' && find ${BASE} -name 'omg-modules*.rpm' && find ${BASE} -name 'omg2-*.rpm' && find ${BASE} -name '*.rpm' | grep -v "omg")
rpmlist2=$(find ${BASE} -name 'omg-cockpit*.rpm')

if test -z "${rpmlist}"; then
    echo "No Packages found."
    exit 1
fi

if [ -e /etc/init.d/omgWebInterface ]; then
    grep -q "exit \$RETVAL" /etc/init.d/omgWebInterface
    if [ "$?" == 0 ]; then
        echo "repairing defective webstart script"
        sed -i 's/exit \$RETVAL/exit 0/' /etc/init.d/omgWebInterface
    fi
fi

if which omgstopall >/dev/null 2>/dev/null; then
    omgstopall
fi

if rpm -q omg2; then
    v=$(rpm -q --qf "%{VERSION}\n" omg2 | sed -e 's/^\([0-9]*[.][0-9]*\).*/\1/')
else
    v=5.1
fi

if [ "$v" == "5.0" ]; then
    rpm --erase --nodeps omg2 omg-modules
    dnf install -y epel-release
    dnf install -y portaudio
fi

if ! rpm -qa | grep -q nodejs; then
    dnf install -y nodejs
fi

rpm -U --force ${rpmlist} || exit 1
rpm -U --force ${rpmlist2} 2>/dev/null

#switching to new web ui
sernumstring=$(/usr/bin/i2cinfo -l | grep -e "OPL\|OPEXT" | head -n 1 | cut -d "," -f 5)
if [ -z $sernumstring ]; then
    let sernum=0
else
    let sernum=$sernumstring
fi

# echo "switching to new webui"
# /usr/bin/omgsetwebui new

exit 0
