# coffeelint: disable=max_line_length

_ = require("lodash")
ko = require "knockout"
Producer = require "../../ProducerClass"
debug = require("../../../shared_modules/logger").debug

bchannelsLib = require "./bchannels"
dchannelsLib = require "./dchannels"

class Channels extends Producer
  constructor: ->
    super "channels", 10 * 1000

    intervalTimeToGetNewData = 5 * 1000

    @running = false

    dchannels = ko.observableArray()
    dchannelsLib.on "new", (channels) ->
      dchannels channels

    bchannels = ko.observableArray()
    bchannelsLib.on "new", (channels) ->
      bchannels channels

    getNewData = false
    run = ->
      # don't run when there are no subscribers
      # return unless @running
      debug "Update channel states"
      dchannelsLib.getChannels()
      bchannelsLib.getChannels()
      dchannelsLib.getStates()
    #run once on load
    run()

    channels = ko.computed ->
      # unless @running
        # debug "Not updating channels, none called start"
        # return
      # only run when all channels are set
      unless dchannels().length and bchannels().length
        debug "not merging channel data because dchannels or bchannels are not set, yet"
        debug "bchannels().length #{bchannels().length} dchannels().length #{dchannels().length}"
        return
      debug "Merging channel data"
      mergedChannels = {}

      bchannelsGrouped = _.groupBy bchannels(), (bchan) ->
        bchanIlid = bchan.split("/")[0]
        return bchanIlid

      _.each dchannels(), (dchan) ->
        dchanIlid = "#{dchan.ilidNumber}.#{dchan.ilidIndex}"

        #add matching bchannels to dchannel
        dchan.bchannels = bchannelsGrouped[dchanIlid]

        dchan.totalLines = _.size(dchan.bchannels)
        dchan.usedLines = parseInt(dchan.CALLS, 10)
        dchan.unusedLines = dchan.totalLines - dchan.usedLines
        dchan.utilization = 100 / dchan.totalLines * dchan.usedLines

        mergedChannels[dchan.name] = dchan

      return mergedChannels

    # send changes
    ko.computed =>
      @sendData ko.toJS channels
    .extend({ rateLimit: 100 })

    @subscribeTo "configurator", (err, status, data) ->
      if data and data.config and data.config.struct_Box and data.config.struct_Box.struct_DChannel
        # debug "from configurator: ", data.config.struct_Box.struct_DChannel
        dchannelsLib.getNewConfig()

    @start = =>
      if @running
        debug "already running"
        return
      debug "starting"
      @running = true
      dchannelsLib.start()
      bchannelsLib.start()
      clearInterval getNewData if getNewData
      getNewData = setInterval run, intervalTimeToGetNewData #5 * 1000

    @stop = =>
      unless @running
        debug "Not running"
        return
      debug "stopping"
      clearInterval getNewData if getNewData
      @running = false
      dchannelsLib.stop()
      bchannelsLib.stop()

  subscribe: =>
    unless @running
      @start()
    super()

module.exports = Channels

return
